/* global goog, nsVmap, oVmap, ol, angular, scope, bootbox, vitisApp, destructor_form */

/**
 * @author: Armand Bahi
 * @Description: Fichier contenant la classe nsVmap.nsToolsManager.BasicSelect
 * cette classe permet l'interrogation de couches
 */
goog.provide('nsVmap.nsToolsManager.BasicSelect');

goog.require('oVmap');

/**
 * @classdesc
 * Class {@link nsVmap.nsToolsManager.BasicSelect}: allow select
 *
 * @constructor
 * @export
 */
nsVmap.nsToolsManager.BasicSelect = function () {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect');
};

/**
 * App-specific directive wrapping the select tools.
 * @return {angular.Directive} The directive specs.
 * @constructor
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectDirective = function () {
    oVmap.log("nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectDirective");
    return {
        restrict: 'A',
        scope: {
            'map': '=appMap',
            'lang': '=appLang',
            'currentAction': '=appAction'
        },
        controller: 'AppBasicselectController',
        controllerAs: 'ctrl',
        bindToController: true,
        templateUrl: oVmap['properties']['vmap_folder'] + '/' + 'template/tools/basicselect.html'
    };
};

/**
 * @ngInject
 * @constructor
 * @param {object} $scope
 * @param {object} $timeout
 * @param {object} $q
 * @returns {nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController}
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController = function ($scope, $timeout, $q) {
    oVmap.log("nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController");

    var this_ = this;

    /**
     * The vMap scope
     */
    $scope['vmapScope'] = oVmap['scope'];

    /**
     * @private
     */
    this.$scope_ = $scope;

    /**
     * @private
     */
    this.$timeout_ = $timeout;

    /**
     * @private
     */
    this.$q_ = $q;

    /**
     * Dummy canceler (use canceler.resolve() for cancel the ajax request)
     * @private
     */
    this.canceler_ = $q.defer();

    /**
     * Parent element
     */
    this.oSelect = oVmap.getToolsManager().getBasicTools().getSelect();

    /**
     * The current properties
     */
    this['properties'] = oVmap['properties'];

    /**
     * The current token
     */
    this['token'] = oVmap['properties']['token'];

    /**
     * Maximum popup mode objects
     * @type integer
     */
    this.limitPopup_ = oVmap['properties']["selection"]["limit_popup"];

    /**
     * Maximum list mode objects
     * @type integer
     */
    this['limitList_'] = oVmap['properties']["selection"]["limit_list"];

    /**
     * @private
     */
    this['allQueryables_'] = false;

    /**
     * contains the current selections
     * @type {array}
     */
    this['aSelections'] = [];

    /**
     * contains the selection before
     * @type {array}
     */
    this.aLastSelections = [];

    /**
     * Contains the information used by the table
     * @type object
     */
    this['tableSelection'] = {};

    /**
     * The selected business object
     */
    this['sSelectedBo'] = '';

    /**
     * Queryable business objects
     */
    this['oQueryableBOs'] = {};

    /**
     * WMS queryable layers
     */
    this['aQueryableLayers'] = {};

    /**
     * WMS GetFeatureInfo tree
     */
    this['aGetFeatureInfoTree'] = {};

    /**
     * Queryable business objects list
     */
    this['aBusinessObjectsList'] = [];

    // Rempli les bo disponibles
    oVmap['scope'].$on('layersChanged', function () {
        setTimeout(function () {
            this_.setQueryableObjects();
        });
    });
    setTimeout(function () {
        this_.setQueryableObjects();
    });

    // Effectue une sélection lors d'un click sur la carte
    this['map'].on('click', function (evt) {
        if (oVmap.getMap().getCurrentAction() === '') {

            // Modale GetFeatureInfo déjà affichée ?
            var bGFIModalOpened = angular.copy($('#getFeatureInfo-result-window').is(":visible"));

            // Re-crée la géométrie en WKT
            var olPoint = new ol.geom.Point(evt.coordinate);
            var EWKTGeometry = oVmap.getEWKTFromGeom(olPoint);

            // Ferme les outils
            oVmap.getToolsManager().getBasicTools().toggleOutTools();

            // Interroge les couches selectionnées
            this.queryByEWKTGeom(EWKTGeometry, olPoint);

            // Interroge les couches sélectionnées par GetFeatureInfo
            if (this['sSelectedBo'] === 'vmap_select_getfeatureinfo') {
                this.queryGetFeatureInfo(evt.coordinate, bGFIModalOpened)
            }
        }
    }, this);

    // Anulle l'édition quand on change d'outil
    oVmap['scope'].$on('toggleOutTools', function () {
        this_.canceler_.resolve();
    });

    // Lors du click sur 'echap', annule les requêtes en cours
    $(document).keydown(function (e) {
        if (e.keyCode === 27) {
            this_.canceler_.resolve();
        }
    });

    // Vide les popups quand on change de carte
    oVmap['scope'].$on('mapChanged', function () {
        this_.removePopups();
    });

    // Événement pour dire à l'appli que this_['sSelectedBo'] a changé
    $scope.$watch('ctrl.sSelectedBo', function (oldVal, newVal) {
        if (!goog.isString(oldVal) || !goog.isString(newVal)) {
            return;
        }
        if (!oldVal.length > 0 || !newVal.length > 0) {
            return;
        }
        if (oldVal === newVal) {
            return;
        }
        oVmap['scope'].$broadcast('selectedBoChanged');
    });
};


// Interrogation par intersection géométrique

/**
 * Call sendQueryByEWKTGeom_ for the selected layers
 * @param {string} EWKTGeometry
 * @param {boolean} useTableFormat true to result in table format, false for popup
 * @param {string|undefined} sQueryBo Business object to query, this.sSelectedBo if undefined
 * @returns {undefined}
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.queryByEWKTGeom = function (EWKTGeometry, olPoint, sQueryBo) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.queryByEWKTGeom');

    var this_ = this;

    // Stoque les paramètres d'interrogation
    this.EWKTGeometry = EWKTGeometry;
    this.olPoint = olPoint;

    $('body').css({"cursor": "wait"});

    // Annule les anciennes requêtes
    this.canceler_.resolve();

    // Réinitialise canceler_
    this.canceler_ = this.$q_.defer();

    // Objet métier sur lequel effectuer la selection
    sQueryBo = goog.isDefAndNotNull(sQueryBo) ? sQueryBo : this['sSelectedBo'];

    // Interroge l'objet métier sélectionné
    if (goog.isDefAndNotNull(this['oQueryableBOs'][sQueryBo])) {

        // Ajoute les filtres des couches associées
        var sLayersFilter = this.getBOLayersFilters_(this['oQueryableBOs'][sQueryBo]);

        this.oSelect.queryByEWKTGeom({
            bo_id: sQueryBo,
            sFilter: sLayersFilter,
            EWKTGeometry: EWKTGeometry,
            canceler: this.canceler_,
            useTableFormat: false,
            buffer: this['oQueryableBOs'][sQueryBo]['bo_selection_buffer'],
            callback: function (data) {
                $('body').css({"cursor": ""});
                this_.replaceSelectionPopup(data, olPoint);
            }
        });
    } else {
        $('body').css({"cursor": ""});
    }
};

/**
 * Get the business object associated layers filter as a API usable string
 * @param {object} oQueryBo
 * @returns {String}
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.getBOLayersFilters_ = function (oQueryBo) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototypeBO.getLayersFilters_');

    var oFilterValues = {};
    var sFilter = "";
    var filter = {
        'relation': 'AND',
        'operators': []
    };

    var oLayerFilter_;
    if (goog.isArray(oQueryBo['layers'])) {
        for (var i = 0; i < oQueryBo['layers'].length; i++) {
            if (oQueryBo['layers'][i].get('is_bo_filtered')) {
                oLayerFilter_ = oQueryBo['layers'][i].get('filter_values_cleared');
                if (goog.isObject(oLayerFilter_)) {
                    goog.object.extend(oFilterValues, oLayerFilter_);
                }
            }
        }
    }

    for (var key in oFilterValues) {

        if (oFilterValues[key] === '') {
            continue;
        }

        // Tableaux (utilisés pour le IN)
        if (goog.isArray(oFilterValues[key])) {

            // Supprime les valeurs vides
            for (var i = oFilterValues[key].length - 1; i >= 0; i--) {
                if(oFilterValues[key][i] === ""){
                    oFilterValues[key].splice(i, 1);
                }
            }

            // Supprime les tableaux vides
            if (oFilterValues[key].length === 0) {
                continue;
            }
        }

        filter['operators'].push({
            'column': key,
            'compare_operator': goog.isArray(oFilterValues[key]) ? 'IN' : '=',
            'value': oFilterValues[key]
        });
    }
    sFilter = JSON.stringify(filter);

    return sFilter;
};


// Interrogation GetFeatureInfo

/**
 * queryGetFeatureInfo function - Lance une requête GetFeatureInfo sur les couches/calcques sélectionnés
 *
 * @param  {array} aCoordinate Coordonnées où a été fait le clic
 * @param  {boolean} bGFIModalOpened Détermine si la modale GetFeatureInfo est déjà ouverte
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.queryGetFeatureInfo = function (aCoordinate, bGFIModalOpened){

    var this_ = this;
    var requestSendCounter = 0;
    var requestResultCounter = 0;

    this['aGetFeatureInfoTree'] = this.getGFITree_(aCoordinate);

    for (var i = 0; i < this['aGetFeatureInfoTree'].length; i++) {
        if (goog.isDefAndNotNull(this['aGetFeatureInfoTree'])) {
            if (goog.isDefAndNotNull(this['aGetFeatureInfoTree'][i]['aSubLayers'])) {
                for (var ii = 0; ii < this['aGetFeatureInfoTree'][i]['aSubLayers'].length; ii++) {
                    if (goog.isDefAndNotNull(this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['url'])) {

                        // Passe l'état en cours
                        this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['state'] = 'loading';

                        // Incrémente le compteur de requêtes parties
                        requestSendCounter++;

                        // Lance la requête
                        ajaxRequest({
                            'method': 'GET',
                            'url': this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['url'],
                            'headers': {
                                'Accept': 'text/html'
                            },
                            'responseType': 'text',
                            'ajaxLoader': true,
                            'success': angular.bind(this, function (i, ii, response) {

                                // Incrémente le compteur de requêtes recues
                                requestResultCounter++;

                                this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['state'] = 'loaded';
                                if (goog.isString(response['data'])) {
                                    if (response['data'].length > 0) {
                                        this['aGetFeatureInfoTree'][i]['aSubLayers'][ii]['result'] = response['data'];
                                        this['aGetFeatureInfoTree'][i]['have_results'] = true;
                                    }
                                }

                                // Affiche la fenêtre resultante
                                if (requestSendCounter === requestResultCounter) {
                                    this.oSelect.showGetFeatureInfoResultWindow(angular.copy(this['aGetFeatureInfoTree']), aCoordinate, bGFIModalOpened);
                                }

                            }, i, ii),
                            'error': function(){

                                // Incrémente le compteur de requêtes recues
                                requestResultCounter++;

                                // Affiche la fenêtre resultante
                                if (requestSendCounter === requestResultCounter) {
                                    this.oSelect.showGetFeatureInfoResultWindow(angular.copy(this['aGetFeatureInfoTree']), aCoordinate, bGFIModalOpened);
                                }
                            }
                        });
                    }
                }
            }
        }
    }
}

/**
 * getGFITree_ function - Retourne un arbre contenant les différents calques/couches avec les URL utilisés pour la sélection GetFeatureInfo
 *
 * @param  {array} aCoordinate Coordonnées où a été fait le clic
 * @return {object}
 * @private
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.getGFITree_ = function (aCoordinate){

    var iViewResolution = this['map'].getView().getResolution();
    var sProjection = this['map'].getView().getProjection().getCode();
    var aGFITree = [];
    var aSubLayers, oLayerNode;

    for (var i = 0; i < this['aQueryableLayers'].length; i++) {
        if (goog.isArray(this['aQueryableLayers'][i].get('queryableSublayers'))) {
            aSubLayers = this['aQueryableLayers'][i].get('queryableSublayers');
            if (aSubLayers.length > 0) {
                oLayerNode = {
                    'name': this['aQueryableLayers'][i].get('name'),
                    'aSubLayers': []
                }
                for (var ii = 0; ii < aSubLayers.length; ii++) {
                    oLayerNode['aSubLayers'].push({
                        'name': aSubLayers[ii],
                        'url': this['aQueryableLayers'][i].getSource().getGetFeatureInfoUrl(aCoordinate, iViewResolution, sProjection, {'INFO_FORMAT': 'text/html', 'QUERY_LAYERS': aSubLayers[ii]})
                    });
                }
                aGFITree.push(oLayerNode);
            }
        }
    }

    return aGFITree;
}


// Ajout de la sélection

/**
 * Add the query result to this['aSelections'] removing the duplicate selections
 * @param {object} opt_options
 * @param {array<object>} opt_options.data
 * @param {string} opt_options.dataType summary|list|form
 * @param {function} opt_options.callback callback to run at the end of the function
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addQueryResult = function (opt_options) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addQueryResult');

    var this_ = this;
    var aData = goog.isDef(opt_options.data) ? opt_options.data : [];
    var dataType = goog.isDef(opt_options.dataType) ? opt_options.dataType : null;
    var callback = goog.isDef(opt_options.callback) ? opt_options.callback : function () {
        return 0;
    };

    this.oSelect.addQueryResult({
        data: aData,
        selection: this['aSelections'],
        dataType: dataType,
        callback: function (aSelections) {
            this_['aSelections'] = aSelections;
            callback.call();
        }
    });
};


// Fonctions mode popup

/**
 * Replace the last popup by a new
 * @param {array} aSelection
 * @param {ol.geom.Point|undefined} olPoint position where to center the popup
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.replaceSelectionPopup = function (aSelection, olPoint) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.replaceSelectionPopup');

    var this_ = this;

    // Vérifie que la feature soit bien présente
    for (var i = aSelection.length - 1; i >= 0; i--) {

        var error = false;
        if (!goog.isDefAndNotNull(aSelection[i]['bo_intersect_geom']) && !goog.isDefAndNotNull(aSelection[i]['olFeature'])) {
            error = true;
        }
        if (goog.isDefAndNotNull(aSelection[i]['olFeature'])) {
            if (!goog.isDefAndNotNull(aSelection[i]['olFeature'].getGeometry())) {
                error = true;
            }
        }
        if (error) {
            $.notify("L'objet sélectionné (" + aSelection[i]['bo_id_field'] + " = " + aSelection[i]['bo_id_value'] + ") n'a pas de géométrie");
            aSelection.splice(i, 1);
        }
    }

    if (aSelection.length === 0) {
        return;
    }

    // Garde en mémoire les anciennes selections pour supprimer leurs popup plus tard
    var newSelection = goog.array.clone(this['aSelections']);
    goog.array.extend(this.aLastSelections, newSelection);

    // Supprime toutes les anciennes popup
    this.removePopups();

    // Ajoute le résultat de dans this['aSelections']
    this_.addQueryResult({
        data: aSelection,
        dataType: 'summary',
        callback: function () {
            this_.closeSelectionPopup(this_.aLastSelections);
            this_.displaySelectionPopup(this_['aSelections'], olPoint);
        }
    });

};

/**
 * Display a selection of popups
 * @param {array} aSelection
 * @param {ol.geom.Point|undefined} olPoint position where to center the popup
 * @private
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displaySelectionPopup = function (aSelection, olPoint) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displaySelectionPopup');

    var this_ = this;
    var scope = this.$scope_;

    for (var i = 0; i < aSelection.length; i++) {

        // Crée la nouvelle popup
        if (goog.isDef(aSelection[i]['mapPopup']))
            aSelection[i]['mapPopup'].remove(false);

        // Si le résultat est un point, alors colle la popup sur le point
        if (goog.isDefAndNotNull(aSelection[i]['olFeature'].getGeometry().getType)) {
            if (aSelection[i]['olFeature'].getGeometry().getType() === 'Point') {
                olPoint = aSelection[i]['olFeature'].getGeometry();
            }
        }

        aSelection[i]['mapPopup'] = new nsVmap.Map.MapPopup(aSelection[i]['olFeature'], olPoint);

        // Ajoute au bonton "closse" la fonction qui rend la popup unlocked quand on la ferme
        aSelection[i]['mapPopup'].HTMLCloser_.selection = aSelection[i];
        aSelection[i]['mapPopup'].HTMLCloser_.addEventListener('click', function () {
            this.selection['locked'] = false;
        }, true);

        // Ajoute le bouton "édition" si have_update_rights est true
        if (aSelection[i]['have_update_rights'] === "true" || aSelection[i]['have_update_rights'] === true) {
            aSelection[i]['mapPopup'].HTMLEdit_ = document.createElement('a');
            aSelection[i]['mapPopup'].HTMLEdit_.href = '#';
            aSelection[i]['mapPopup'].HTMLEdit_.className = 'ol-popup-edit icon-draw';
            aSelection[i]['mapPopup'].HTMLButtonsContainer_.appendChild(aSelection[i]['mapPopup'].HTMLEdit_);

            // Ajoute l'interraction sur le bouton "édition"
            aSelection[i]['mapPopup'].HTMLEdit_.scope = this.$scope_;
            aSelection[i]['mapPopup'].HTMLEdit_.selection = aSelection[i];
            aSelection[i]['mapPopup'].HTMLEdit_.addEventListener('click', function () {
                var selection = this.selection;
                scope.$apply(function () {
                    scope['ctrl'].displayEditFrom(selection);
                });
            }, true);
        }

        // Ajoute le bouton "fiche descriptive" si have_card_rights est true
        if (aSelection[i]['have_card_rights'] === "true" || aSelection[i]['have_card_rights'] === true) {
            aSelection[i]['mapPopup'].HTMLCard_ = document.createElement('a');
            aSelection[i]['mapPopup'].HTMLCard_.href = '#';
            aSelection[i]['mapPopup'].HTMLCard_.className = 'ol-popup-card glyphicon glyphicon-file';
            aSelection[i]['mapPopup'].HTMLButtonsContainer_.appendChild(aSelection[i]['mapPopup'].HTMLCard_);

            // Ajoute l'interraction sur le bouton "fiche descriptive"
            aSelection[i]['mapPopup'].HTMLCard_.scope = this.$scope_;
            aSelection[i]['mapPopup'].HTMLCard_.selection = aSelection[i];
            aSelection[i]['mapPopup'].HTMLCard_.addEventListener('click', function () {
                var selection = this.selection;
                scope.$apply(function () {
                    scope['ctrl'].displayObjectCard(selection);
                });
            }, true);
        }

        // Ajout du contennu
        aSelection[i]['mapPopup'].displayMessage('<span class="vmap-popup-title">' + this.oSelect.parseFormTile(aSelection[i]['bo_summarytitle'], aSelection[i]['bo_summary']) + '</span>');
        aSelection[i]['mapPopup'].addMessageTable(aSelection[i]['bo_summary']);
        if (goog.isDef(aSelection[i]['bo_image_path']))
            aSelection[i]['mapPopup'].addPhoto(aSelection[i]['bo_image_path']);


        // Ajoute le bouton "Modifier la géométrie" si have_update_rights = true
        if (aSelection[i]['have_update_rights'] === "true" || aSelection[i]['have_update_rights'] === true) {
            aSelection[i]['mapPopup'].addAction({
                'content': '<span class="icon-polygon"></span> Modifier la géométrie',
                'event': angular.bind(this, function(i){
                    scope.$apply(function () {
                        scope['ctrl'].editFeature(scope['ctrl']['aSelections'][i]);
                    });
                }, i)
            });
        }

        // Ajoute le bouton "Supprimer l'objet" si have_delete_rights = true
        if (aSelection[i]['have_delete_rights'] === "true" || aSelection[i]['have_delete_rights'] === true) {
            aSelection[i]['mapPopup'].addAction({
                'content': '<span class="icon-trash"></span> Supprimer l\'objet',
                'event': angular.bind(this, function(i){
                    scope.$apply(function () {
                        scope['ctrl'].deleteObject(scope['ctrl']['aSelections'][i]);
                    });
                }, i)
            });
        }

        // Ajoute le bouton "Ajouter au panier"
        if (!oVmap['properties']['is_mobile']) {
            aSelection[i]['mapPopup'].addAction({
                'content': '<span class="icon-shopping_basket"></span> Ajouter au panier',
                'event': angular.bind(this, function(i){
                    scope.$apply(function () {
                        scope['ctrl'].addToCard(scope['ctrl']['aSelections'][i]);
                    });
                }, i)
            });
        }

        // Google street view
        if (oVmap['properties']['selection']['street_view'] !== false) {
            var geometry = aSelection[i]['olFeature'].getGeometry();
            var pointCoords = geometry.getFirstCoordinate();
            var proj = this['map'].getView().getProjection().getCode();
            pointCoords = ol.proj.transform(pointCoords, proj, 'EPSG:4326');
            var url = 'http://maps.google.com/maps?q=&layer=c&cbll=' + pointCoords[1] + ',' + pointCoords[0] + '';
            aSelection[i]['mapPopup'].addAction({
                'content': '<span class="icon-street-view"></span> Google Street View',
                'event': function () {
                    var win = window.open(url, '_blank');
                    win.focus();
                }
            });
        }

        aSelection[i]['mapPopup'].addAction({
            'content': '<span class="icon-link_url"></span> Lien vers cet objet' ,
            'event': angular.bind(this, function(i){
                scope.$apply(function () {
                    // generate url for this object;
                    var sUrl = this_['properties']['web_server_name'] + "/" + this_['properties']['application'];

                    if (goog.isDefAndNotNull(this_['properties']['environment'])){
                        if (this_['properties']['environment'] != ""){
                            sUrl += "_" + this_['properties']['environment'];
                        }
                    }

                    sUrl += "?mode_id=vmap&";

                    var oMapCatalog = oVmap.getMapManager().getMapCatalog();
                    var iMapId = oMapCatalog["maps"][oMapCatalog["usedMap"]]["map_id"];

                    sUrl += "map_id=" + iMapId + "&";
                    sUrl += "bo_id=" + aSelection[i]["bo_type"] + "&";
                    sUrl += "ids=" + aSelection[i]["bo_id_value"];

                    //copy url to clipboard
                    var textArea = document.createElement("textarea");
                    textArea.value = sUrl;
                    document.body.appendChild(textArea);
                    textArea.focus();
                    textArea.select();

                    try {
                        var successful = document.execCommand('copy');
                        if (successful){
                            $.notify('L\'URL d\'accés à cette objet a été copié dans votre presse-papier : \n' + sUrl, 'success');
                        } else {
                            $.notify('L\'URL d\'accés à cette objet n\'a pas pu être copié dans votre presse-papier : \n' + sUrl, 'error');
                        }
                    } catch (err) {
                        $.notify('L\'URL d\'accés à cette objet n\'a pas pu être copié dans votre presse-papier : \n' + sUrl, 'error');
                        console.error('Fallback: Oops, unable to copy', err);
                    }
                    document.body.removeChild(textArea);
                });
            }, i)
        });
    }

    // Ajoute les résultats
    goog.array.insertArrayAt(this['aSelections'], aSelection, -1);

    // supprime les doublons
    this['aSelections'] = this.oSelect.removeDuplicateSelections(this['aSelections']);
};

/**
 * Center the map on a popup
 * @param {object} selection
 * @param {object} selection.olFeature
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.centerMapOnPopup = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.basicSelectController.centerMapOnPopup');

    if (!goog.isDefAndNotNull(selection['olFeature'])) {
        console.error("selection['olFeature'] undefined:", selection['olFeature'])
        return 0;
    }

    var geom = selection['olFeature'].getGeometry();

    if (!goog.isDefAndNotNull(geom)) {
        console.error("selection['olFeature'].getGeometry() not defined");
        return 0;
    }

    if (geom.getType() === 'Point') {
        this['map'].getView().setCenter(geom.getCoordinates());
    } else if (geom.getType() === 'MultiPoint') {
        if (geom.getPoints().length === 1) {
            this['map'].getView().setCenter(geom.getPoint(0).getCoordinates());
        } else {
            this['map'].getView().fit(geom);
        }
    } else {
        this['map'].getView().fit(geom);
    }

};


// Foncitons Édition/consultation formReader

/**
 * Display the object card
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayObjectCard = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayObjectCard');
    this.oSelect.displayObjectCard(selection);
};

/**
 * Display the edit form
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayEditFrom = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.displayEditFrom');
    var this_ = this;
    var sQueryBo = angular.copy(selection['bo_type']);
    this.oSelect.displayEditFrom(selection, function () {
        this_.queryByEWKTGeom(this_.EWKTGeometry, this_.olPoint, sQueryBo);
    });
};


// Édition de la géométrie


/**
 * Start the geometry edition
 * @param {object} selection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.editFeature = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.editFeature');

    // Ferme l'ensemble des popups
    this.closeSelectionPopup();

    this.oSelect.editFeature(selection);
};


// Suppression

/**
 * Suggest if realy want to delete the object and call submitDeleteObject
 * @param {object} selection
 * @param {object} selection.bo_type
 * @param {object} selection.bo_id_value
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.deleteObject = function (selection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.basicSelectController.deleteObject');

    var this_ = this;

    this.oSelect.deleteObject(selection, function () {

        // Ferme les popup
        this_.closeSelectionPopup();

        // Supprime de la selection en cours
        this_.removeFromTableSelection(selection);
    });
};


// Utilitaires


/**
 * setQueryableBos function - Définit les Calques et objets métiers interrogeables
 *
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.setQueryableObjects = function () {

    var this_ = this;

    this_.$scope_.$applyAsync(function () {

        // rempli this.oQueryableBOs et aQueryableBOs
        this_['aQueryableLayers'] = oVmap.getMapManager().getQueryableGetFeatureInfoLayers(true);
        this_['oQueryableBOs'] = oVmap.getMapManager().getQueryableBusinessObjects(true);
        this_['aQueryableBOs'] = oVmap.getMapManager().getQueryableBusinessObjectsAsArray(true);
        this_['aBusinessObjectsList'] = Object.keys(this_['oQueryableBOs']);

        // Élément sélectionné par défaut
        if (!goog.isDefAndNotNull(this_['oQueryableBOs'][this_['sSelectedBo']])) {

            var iSmallestIndex = 1000000;
            var sFirstBo = '';

            for (var bo in this_['oQueryableBOs']) {
                if (this_['oQueryableBOs'][bo]['bo_index'] < iSmallestIndex) {
                    iSmallestIndex = this_['oQueryableBOs'][bo]['bo_index'];
                    sFirstBo = this_['oQueryableBOs'][bo]['bo_id'];
                }
            }
            this_['sSelectedBo'] = sFirstBo;
        }
        if (oVmap['properties']['selection']['enable_getfeatureinfo'] === true) {
            if (goog.isArray(this_['aQueryableLayers'])) {
                if (this_['aQueryableLayers'].length > 0) {
                    this_['sSelectedBo'] = 'vmap_select_getfeatureinfo';
                }
            }
        }
    });
}

/**
 * Remove the popups and empty this['aSelections']
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removePopups = function () {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removePopups');

    // Supprime toutes les anciennes popup
    this.closeSelectionPopup();

    // Supprime toutes les anciennes selections
    this.removeSelections();
};

/**
 * Remove all the selections from this['aSelections']
 * @param {string|undefined} bo_id if defined, the function will only remove the selections witch belong the business object
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removeSelections = function (bo_id) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removeSelections');

    // Supprime toutes les selections
    for (var i = this['aSelections'].length - 1; i >= 0; i--) {

        // Si bo_id est défini, supprime uniquement les selections correspondantes au bo_id
        if (goog.isDefAndNotNull(bo_id)) {
            if (this['aSelections'][i]['bo_type'] === bo_id)
                goog.array.removeAt(this['aSelections'], i);
        } else {
            goog.array.removeAt(this['aSelections'], i);
        }
    }
};

/**
 * Remove an element from this.aSelections
 * @param {type} selection
 * @returns {undefined}
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.removeFromTableSelection = function (selection) {

    for (var i = this['aSelections'].length - 1; i >= 0; i--) {
        if (angular.equals(this['aSelections'][i], selection)) {
            this['aSelections'].splice(i, 1);
        }
    }
};

/**
 * Remove the popups from aSelection or this['aSelections']
 * @param {array|undefined} aSelection
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.closeSelectionPopup = function (aSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.closeSelectionPopup');

    aSelection = goog.isDefAndNotNull(aSelection) ? aSelection : this['aSelections'];

    for (var i = 0; i < aSelection.length; i++) {
        if (goog.isDef(aSelection[i]['mapPopup']))
            aSelection[i]['mapPopup'].remove();
    }

};

/**
 * Add the selected elements from the table to the card
 * @param {object} oSelection
 * @export
 */
nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.addToCard = function (oSelection) {
    oVmap.log('nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController.prototype.tableAddToCard');
    this.oSelect.addToCard([oSelection]);
};


// Définit la directive et le controller
oVmap.module.directive('appBasicselect', nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectDirective);
oVmap.module.controller('AppBasicselectController', nsVmap.nsToolsManager.BasicSelect.prototype.basicSelectController);
